package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.params.BasicCampusParams;
import com.qmth.distributed.print.business.bean.result.BasicCampusResult;
import com.qmth.distributed.print.business.entity.BasicCampus;
import com.qmth.distributed.print.business.mapper.BasicCampusMapper;
import com.qmth.distributed.print.business.service.BasicCampusService;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

/**
 * <p>
 * 基础校区表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2021-08-02
 */
@Service
public class BasicCampusServiceImpl extends ServiceImpl<BasicCampusMapper, BasicCampus> implements BasicCampusService {
    @Resource
    private BasicCampusMapper basicCampusMapper;

    @Override
    public List<BasicCampusResult> basicCampusList(Long schoolId) {
        return basicCampusMapper.findBasicCampusResult(schoolId);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Long saveBasicCampus(BasicCampusParams basicCampusParams) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Long schoolId = sysUser.getSchoolId();
        Long orgId = sysUser.getOrgId();

        String campusName = basicCampusParams.getCampusName();
        String campusCode = basicCampusParams.getCampusCode();
        if (!SystemConstant.strNotNull(campusCode)){
            throw ExceptionResultEnum.ERROR.exception("缺少校区编号");
        }
        Long campusId;
        if (SystemConstant.longNotNull(basicCampusParams.getId())){
            // 编辑
            campusId = basicCampusParams.getId();
            UpdateWrapper<BasicCampus> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda()
                    .set(BasicCampus::getCampusName,campusName)
                    .set(BasicCampus::getOrgId,orgId)
                    .set(BasicCampus::getSchoolId,schoolId)
                    .set(BasicCampus::getUpdateId,sysUser.getId())
                    .eq(BasicCampus::getId,campusId);
            this.update(updateWrapper);
        }else {
            // 新增
            campusId = SystemConstant.getDbUuid();
            BasicCampus basicCampus = new BasicCampus();
            basicCampus.setId(campusId);
            basicCampus.setCampusCode(campusCode);
            basicCampus.setCampusName(campusName);
            basicCampus.setOrgId(orgId);
            basicCampus.setEnable(true);
            basicCampus.setCreateId(sysUser.getId());
            basicCampus.setSchoolId(schoolId);
            this.save(basicCampus);
        }
        return campusId;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Boolean removeBasicCampusById(Long id) {
        UpdateWrapper<BasicCampus> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(BasicCampus::getEnable,false).eq(BasicCampus::getId,id);
        return this.update(updateWrapper);
    }
}
